/* 
SUPPORT: sinruk Kulkarni, Rhuta
*/ 
/****************************************************************************** 
 * Job:             Load_STG2DM_Product_dm                                    * 
 *                                                                            * 
 * Source Tables:   STG_PRODUCT - DIDM_STG.STG_PRODUCT                        * 
 *                  PRODUCT_DM - DI_DM.PRODUCT_DM                             * 
 *                  STG_PRODUCT_HIERARCHY -                                   * 
 *                   DIDM_STG.STG_PRODUCT_HIERARCHY                           * 
 *                  PRODUCT_DM - DI_DM.PRODUCT_DM                             * 
 * Target Tables:   PRODUCT_DM - DI_DM.PRODUCT_DM                             * 
 *                  PROD_HIER_SK - DI_MON.PROD_HIER_SK                        * 
 *                                                                            * 
 * Generated on:    Monday, May 17, 2010 2:16:37 PM IST                       * 
 * Generated by:    sinruk@row                                                * 
 ******************************************************************************/ 

/* General macro variables  */ 
%let jobID = %quote(A5W0V4GZ.BF0004LO);
%let etls_jobName = %nrquote(Load_STG2DM_Product_dm);
%let etls_userID = %nrquote(sinruk@row);

/* Setup to capture return codes  */ 
%global job_rc trans_rc sqlrc; 
%let sysrc = 0;
%let job_rc = 0;
%let trans_rc = 0;
%let sqlrc = 0;
%global syserr; 

%macro rcSet(error); 
   %if (&error gt &trans_rc) %then 
      %let trans_rc = &error;
   %if (&error gt &job_rc) %then 
      %let job_rc = &error;
%mend rcSet; 

%macro rcSetDS(error); 
   if &error gt input(symget('trans_rc'),12.) then 
      call symput('trans_rc',trim(left(put(&error,12.))));
   if &error gt input(symget('job_rc'),12.) then 
      call symput('job_rc',trim(left(put(&error,12.))));
%mend rcSetDS; 

/* Create metadata macro variables */
%let IOMServer      = %nrquote(MIMain);

/* Setup for capturing job status  */ 
%let etls_startTime = %sysfunc(datetime(),datetime.);
%let etls_recordsBefore = 0;
%let etls_recordsAfter = 0;
%let etls_lib = 0;
%let etls_table = 0;

%global etls_debug; 
%macro etls_setDebug; 
   %if %str(&etls_debug) ne 0 %then 
      OPTIONS MPRINT%str(;); 
%mend; 
%etls_setDebug; 

/*---- Start of Pre-Process Code  ----*/ 

/**/
%etl_initjobparam (monlib=di_mon, jobname=%nrquote(&etls_jobName));
/**/
%etl_initchangeflags;

/*---- End of Pre-Process Code  ----*/ 

%rcSet(&syserr); 
%rcSet(&sqlrc); 

/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            UW - Check Dependent Tables             A5W0V4GZ.BH00099Q * 
 * Transform:       User Written                                              * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Table:    STG_PRODUCT - DIDM_STG.STG_PRODUCT      A5W0V4GZ.BB0004OK * 
 * Target Table:    User Written - DIDM_STG.STG_PRODUCT     A5W0V4GZ.BK0007IU * 
 *                                                                            * 
 * User Written:    SourceCode                                                * 
 *============================================================================*/ 

/*get max prod level for standard hierarchy*/
%macro etl_getprodlvl;

   %global g_max_prod_lvl cnt_assoc_cd;
   %global g_valid_prod_lvls;
   %let g_valid_prod_lvls=0;
   %let cnt_assoc_cd = 0;
   proc sql noprint;
      select max(prod_lvl) into :g_max_prod_lvl
      from di_dm.product_lvl
      where PROD_HIER_ASSOC_CD = 1;
   quit;
   %if &g_max_prod_lvl eq . %then %do;
       %let etl_msg=%etl_msg(mid=5013,args=Prod_lvl PRODUCT_LVL);
       %put &etl_msg;
       %etl_abort;
   %end;
   %else %do;
      proc sql noprint;
         select prod_lvl into :g_valid_prod_lvls separated by ','
         from di_dm.product_lvl
         where PROD_HIER_ASSOC_CD = 1;
      quit;
   %end;

   /* Check whether standard or any other alternate hierarchy exist */
   proc sql noprint;
     select count(distinct(PROD_HIER_ASSOC_CD)) into :cnt_assoc_cd
     from di_dm.product_lvl;
   quit;
   
%mend etl_getprodlvl;

%etl_getprodlvl;

%global etls_recordsBefore1 PROD_ID_1_LVL;

%macro create_global_var;

   %let etls_recordsBefore1=%di_util_nobs(DI_DM.PRODUCT_DM);
   %if &etls_recordsBefore1 > 0 %then %do;
      proc sql noprint;
      select PROD_ID into :PROD_ID_1_LVL 
      from DI_DM.PRODUCT_DM 
      where STD_HIER_PROD_LVL=1;
      quit;
   %end;
   %let PROD_ID_1_LVL = &PROD_ID_1_LVL;
   %put PROD_ID_1_LVL = &PROD_ID_1_LVL;

%mend create_global_var;

%create_global_var;

/********************************************************************************************
  Restrict future recycling of product. 
  If a product's DISC_DT is in future. Then the same PROD_ID should not be recycled. 
  To restrict the future recycling, check for re-cycled cases where incoming
  product's INTR_DT > APP_DT and then move that PROD_ID to exception table. 
  Find the value of parameter APP_DT. If the value is missing then use today's date.
********************************************************************************************/

%macro find_app_dt; 
%global apln_dt app_dt;
%let apln_dt = 0;
%let app_dt = 0;
 /* Select APP_DATE from DI_DM.GLOBAL_SETTING table */
  proc sql noprint;
     select SPEC_VALUE into : apln_dt
     from di_dm.global_setting
     where upcase(SPEC_NM) = "APP_DATE";
  quit;
  %if &apln_dt = %str() %then %do; 
      %let app_dt = %sysfunc(today());
  %end;
  %else %do;
     data _null_;
       app_date = input(symget('apln_dt'), mmddyy10.);
      call symputx('app_dt', app_date);
     run;
  %end;
  %put "Application Date = " &app_dt;
%mend;

%find_app_dt;

%rcSet(&syserr); 
%rcSet(&sqlrc);

/*---- End of User Written Code  ----*/ 

/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            UW - Remove Duplicate                   A5W0V4GZ.BH00099S * 
 * Transform:       User Written                                              * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Table:    Sort - work.STG_PROD_SORTED             A5W0V4GZ.BK0007IV * 
 * Target Table:    User Written - work.STG_PROD_NODUP      A5W0V4GZ.BK0007IW * 
 *                                                                            * 
 * User Written:    SourceCode                                                * 
 *============================================================================*/ 

%let transformID = %quote(A5W0V4GZ.BH00099S);
%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.); 

/*---- Start of User Written Code  ----*/ 

proc sort data=didm_stg.stg_product out=stg_prod_sorted;
   by prod_id change_flg prod_introduction_dt;
run;   

data work.stg_prod_nodup (drop=lvl_1_count etls_exception_cond etl_error_jobruntime)
     work.duplicate_prod(keep=prod_id std_hier_prod_lvl prod_introduction_dt prod_discontinued_dt 
                            change_flg etls_exception_cond etl_error_jobruntime) ;

   retain lvl_1_count 0;
   length etls_exception_cond $300. etl_error_jobruntime 8;
   set stg_prod_sorted;
   by prod_id change_flg prod_introduction_dt;

   if std_hier_prod_lvl ne 1 & not (last.change_flg) then do;
      etls_exception_cond = cats(etls_exception_cond,'0a'x," * %etl_msg(mid=5303,args=PROD_ID)"); 
   end;

   if std_hier_prod_lvl = 1 then do;
      if lvl_1_count=0 then lvl_1_count+1;
      else etls_exception_cond = cats(etls_exception_cond,'0a'x," * %etl_msg(mid=5303,args=PROD_LVL=1)");
   end;
   if not missing(etls_exception_cond) then do;
      etl_error_jobruntime = datetime();
      output duplicate_prod;
   end;
   else output stg_prod_nodup;
run;

proc append base=di_mon.stg_product_x data=work.duplicate_prod force nowarn;
run;

%rcSet(&syserr); 
%rcSet(&sqlrc);

   
/*---- End of User Written Code  ----*/ 

/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            Lookup                                  A5W0V4GZ.BH00099U * 
 * Transform:       Lookup                                                    * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Tables:   User Written - work.VALID_RECS          A5W0V4GZ.BK0007IW * 
 *                  PRODUCT_DM - DI_DM.PRODUCT_DM           A5W0V4GZ.BB0004JH * 
 *                  STG_PRODUCT_HIERARCHY -                 A5W0V4GZ.BB0004ON * 
 *                   DIDM_STG.STG_PRODUCT_HIERARCHY                           * 
 * Target Table:    Lookup - work.STG_PROD_DETAILS          A5W0V4GZ.BK0007IY * 
 *                                                                            * 
 * User Written:    SourceCode                                                * 
 *============================================================================*/ 

%di_etl_keep_work_table(stg_prod_nodup);
/*---- End of Pre-Process Code  ----*/ 

%rcSet(&syserr); 
%rcSet(&sqlrc); 

%macro product_data_validation;

   %local prod_count prod_hier_count prev_prod_data_num;
   %let runTime = %sysfunc(datetime()); 
   proc datasets lib = work nolist nowarn memtype = (data view);
      delete etls_temp_lookupview0 prev_prod_data;
   quit;

   data etls_temp_lookupview0;
      set di_dm.product_dm;
      by prod_id;
      if last.prod_id;
      keep prod_hier_sk prod_id prod_introduction_dt prod_discontinued_dt std_hier_prod_lvl;
      rename prod_introduction_dt = dm_sal_intr_dt 
             prod_discontinued_dt = dm_sal_disc_dt 
             std_hier_prod_lvl = dm_prod_lvl;
   run;

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete etls_temp_lookupview1;
   quit;

   proc sql; 
      create table etls_temp_lookupview1 as 
      select prod_id, prod_lvl from didm_stg.stg_product_hierarchy
      where prod_hier_assoc_cd = 1;
   quit;

   data prev_prod_data;
      set di_dm.product_dm;
      by prod_id;
      if NOT (first.prod_id eq 1 and last.prod_id eq 1);
      prev_prod_id=lag(prod_id);
      prev_prod_intr_dt=lag(prod_introduction_dt);
      prev_prod_disc_dt=lag(prod_discontinued_dt);
      if last.prod_id and prev_prod_id=prod_id ;
      keep prod_hier_sk prev_prod_intr_dt prev_prod_disc_dt;
   run;

   proc datasets lib = work nolist nowarn memtype = (data view);
      delete stg_prod_details update_prod_disc_dt missing_prod_sk;
   quit;

   %let prod_count = %di_util_nobs(work.etls_temp_lookupview0);
   %let prod_hier_count = %di_util_nobs(work.etls_temp_lookupview1);
   %let prev_prod_data_num = %di_util_nobs(prev_prod_data);

   /*---- Begin DATA step to perform lookups  ----*/ 
   DATA  work.STG_PROD_DETAILS
            ( keep = PROD_ID PROD_NM STD_HIER_PROD_LVL PROD_DESC PROD_INTRODUCTION_DT PROD_DISCONTINUED_DT VOLUME 
                     VOLUME_BASE_UOM BRAND_NM MODEL_PROD_FLG CHANGE_FLG PROD_HIER_SK DM_SAL_INTR_DT 
                     DM_SAL_DISC_DT DM_PROD_LVL STG_STD_PROD_LVL PREV_PROD_INTR_DT PREV_PROD_DISC_DT)
         missing_prod_sk ( keep = PROD_ID PROD_NM STD_HIER_PROD_LVL PROD_DESC PROD_INTRODUCTION_DT PROD_DISCONTINUED_DT VOLUME 
                     VOLUME_BASE_UOM BRAND_NM MODEL_PROD_FLG CHANGE_FLG PROD_HIER_SK DM_SAL_INTR_DT 
                     DM_SAL_DISC_DT DM_PROD_LVL STG_STD_PROD_LVL PREV_PROD_INTR_DT PREV_PROD_DISC_DT)
         update_prod_disc_dt ( keep = PROD_ID PROD_INTRODUCTION_DT DM_SAL_INTR_DT DM_SAL_DISC_DT );

      attrib PROD_ID length=$32
             PROD_NM length=$40
             STD_HIER_PROD_LVL length=4
             PROD_DESC length=$100
             PROD_INTRODUCTION_DT length=8 format=DATE9.
             PROD_DISCONTINUED_DT length=8 format=DATE9.
             VOLUME length=8 format=NLNUM12.2
             VOLUME_BASE_UOM length=$20
             BRAND_NM length=$40             
             MODEL_PROD_FLG length=4
             CHANGE_FLG length=$1
             ETLS_EXCEPTION_COND length=$300
             PROD_HIER_SK length=6
             DM_SAL_INTR_DT length=8 format=DATE9.
             DM_SAL_DISC_DT length=8 format=DATE9.
             DM_PROD_LVL length=4
             STG_STD_PROD_LVL length=4
             PROD_LVL length= 4
             PREV_PROD_INTR_DT length=8 format=DATE9.
             PREV_PROD_DISC_DT length=8 format=DATE9.
             ; 

      target_val = 0; 
      if (_n_ = 1) then do; 
         prod_count = symget('prod_count');

         if prod_count gt 0 then exponent0 = ceil(log2(prod_count));
         else exponent0 = 8;

         prod_hier_count = symget('prod_hier_count');

         if prod_hier_count gt 0 then exponent1 = ceil(log2(prod_hier_count));
         else exponent1 = 8;

         prev_prod_data_num = symget('prev_prod_data_num');
         
         if prev_prod_data_num gt 0 then exponent2 = ceil(log2(prev_prod_data_num));
         else exponent2=8;
         
         declare hash h0(dataset: "etls_temp_lookupview0", hashexp: exponent0);
         h0.defineKey( "PROD_ID");
         h0.defineData( "PROD_HIER_SK",
                        "DM_SAL_INTR_DT",
                        "DM_SAL_DISC_DT",
                        "DM_PROD_LVL");
         h0.defineDone();

         declare hash h1(dataset: "etls_temp_lookupview1", hashexp: exponent1);
         h1.defineKey( "PROD_ID");
         h1.defineData( "PROD_LVL");
         h1.defineDone();

         declare hash h2(dataset: "prev_prod_data", hashexp: exponent2);
         h2.defineKey( "PROD_HIER_SK");
         h2.defineData( "PREV_PROD_INTR_DT" ,"PREV_PROD_DISC_DT");
         h2.defineDone();

         call missing (PROD_ID, PROD_HIER_SK, DM_SAL_INTR_DT, DM_SAL_DISC_DT, DM_PROD_LVL, 
                       STG_STD_PROD_LVL, PROD_LVL, PREV_PROD_INTR_DT, PREV_PROD_DISC_DT);    
      end; 

      set stg_prod_nodup end = eof;
      by prod_id;

      rc0 = h0.find();

      rc1 = h1.find();
      STG_STD_PROD_LVL = PROD_LVL;

      /* lookup from prev_prod_data */
      rc2 = h2.find();
      
      if ( rc0=0 ) then do; 
         if (NOT MISSING (PROD_HIER_SK)AND UPCASE(CHANGE_FLG)= UPCASE("&etl_insert_flg") AND 
             NOT MISSING(PROD_INTRODUCTION_DT) AND MISSING (DM_SAL_DISC_DT) AND 
             NOT MISSING(DM_SAL_INTR_DT) AND ((PROD_INTRODUCTION_DT - 1) gt dm_sal_intr_dt) 
             AND (std_hier_prod_lvl ne 1)) 
         then do;        
           DM_SAL_DISC_DT = PROD_INTRODUCTION_DT - 1;
           output update_prod_disc_dt;
         end;
 
         output work.STG_PROD_DETAILS; 
         return; 
      end; 
      else do;     
         if (rc0 ^= 0) then do; 
               call missing (PROD_HIER_SK, DM_SAL_INTR_DT, DM_SAL_DISC_DT, DM_PROD_LVL); 
               target_val = 1;
               missing_prod = 1; 
         end; 

         if (rc1 ^= 0) then do; 
               call missing (STG_STD_PROD_LVL); 
               target_val = 1; 
         end; 

         if (target_val eq 1) then 
            output work.STG_PROD_DETAILS;              
         if (missing_prod eq 1) then 
            output missing_prod_sk; 
      end; 
      
   run; 

   %rcSet(&syserr);
   %rcSet(&sqlrc);

%mend;

%product_data_validation;

/********************************************************************************************
   If multiple products are coming for the first time with different CHANGE_FLG then
   1. If both records have CHANGE_FLG = I & U, and it is a new product and have same intro_dt then 
      we load the record with CHANGE_FLG = U in product-dm. Reject the record whose CHANGE_FLG = I.
   2. If both I and U come, it is a new product and have different intro_dt then 
      record with new intro_dt will be loaded. Reject the older intro_dt.   
*********************************************************************************************/
%macro dup_prods ;
/********************************************************************************************
  Table "missing_prod_sk" contains all the new products. Sort table by PROD_ID,
  PROD_INTRODUCTION_DT & CHANGE_FLG. 
  Hence last record will always be a record with CHANGE_FLG = U. 
  Move all records except last.PROD_ID to the table "duplicate_prod". Append errornous records
  to di_mon.stg_product_x.
  Delete all the records from table "STG_PROD_DETAILS", which exist in "duplicate_prod" based
  on PROD_ID, PROD_INTRODUCTION_DT & CHANGE_FLG. 
*********************************************************************************************/
  %if %di_util_nobs(missing_prod_sk) gt 0 %then %do;

     proc sort data = missing_prod_sk out = srt_missing_prod_sk;
       by prod_id PROD_INTRODUCTION_DT change_flg;
     run;


     data duplicate_prod (keep=prod_id std_hier_prod_lvl prod_introduction_dt prod_discontinued_dt 
                               change_flg etls_exception_cond etl_error_jobruntime) ;
       length etls_exception_cond $300. etl_error_jobruntime 8;
       set srt_missing_prod_sk;
       by PROD_ID;
       if not (last.PROD_ID) then do;
          ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5357)"),$300.); 
       end;
       if not missing(etls_exception_cond) then do;
          etl_error_jobruntime = datetime();
          output duplicate_prod;
       end;
     run;
  
     %if %di_util_nobs(duplicate_prod) gt 0 %then %do;

        proc append base=di_mon.stg_product_x 
                    data=work.duplicate_prod force nowarn;
        run;

        proc sql;
          delete from STG_PROD_DETAILS as val
          where prod_id in (select prod_id from duplicate_prod as err
                            where val.prod_id = err.prod_id
                            and   val.PROD_INTRODUCTION_DT = err.PROD_INTRODUCTION_DT
                            and   upcase(val.change_flg) = upcase(err.change_flg));
        quit;
     %end;
  %end; /* No new prods exist i.e. missing_prod_sk is empty. */ 
%mend;

%dup_prods;
/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            UW Data Validation                      A5EEB9WX.BI0000A9 * 
 * Transform:       UW Data Validation                                        * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Table:    Lookup - work.STG_PROD_DETAILS   A5EEB9WX.BK000092 * 
 * Target Table:    Data Validation -                       A5EEB9WX.BK000093 * 
 *                   work.STG_VALID_PRODUCT                                   * 
 *============================================================================*/ 

%di_etl_keep_work_table(STG_PROD_DETAILS update_prod_disc_dt);

%macro etls_Validate(etls_Source=); 

   %let runTime = %sysfunc(datetime()); 
   
   proc datasets lib = work nolist nowarn memtype = (data view);
      delete STG_VALID_PRODUCT etls_Errors CHECK_RECYCLED_PRODS;
   quit;
   
   /*---- Begin checks for Custom, Missing Values, and Invalid Values  ----*/ 
   data work.STG_VALID_PRODUCT
         (keep = PROD_ID PROD_NM STD_HIER_PROD_LVL PROD_DESC PROD_INTRODUCTION_DT 
                 PROD_DISCONTINUED_DT VOLUME VOLUME_BASE_UOM BRAND_NM 
                 MODEL_PROD_FLG CHANGE_FLG PROD_HIER_SK CREATED_DTTM 
                 STD_HIER_LEAF_NODE_FLG)
                 
        work.etls_Errors
         (keep = PROD_ID STD_HIER_PROD_LVL PROD_INTRODUCTION_DT PROD_DISCONTINUED_DT 
                 CHANGE_FLG ETLS_EXCEPTION_COND ETL_ERROR_JOBRUNTIME)
                 
        work.CHECK_RECYCLED_PRODS
         (keep= PROD_ID PROD_NM STD_HIER_PROD_LVL PROD_DESC PROD_INTRODUCTION_DT 
                PROD_DISCONTINUED_DT VOLUME VOLUME_BASE_UOM BRAND_NM 
                MODEL_PROD_FLG CHANGE_FLG PROD_HIER_SK CREATED_DTTM 
                STD_HIER_LEAF_NODE_FLG DM_SAL_INTR_DT DM_SAL_DISC_DT PREV_PROD_INTR_DT PREV_PROD_DISC_DT);
                
      /* Define lengths for columns with longer lengths in target  */ 
      attrib STD_HIER_LEAF_NODE_FLG length = 4 ETL_ERROR_JOBRUNTIME length=8 format=DATETIME16. 
             ETLS_EXCEPTION_COND length=$300 CREATED_DTTM length=8 chk_recycle_flg length=4;
      
      set STG_PROD_DETAILS end=eof;

      chk_recycle_flg = 0;
      
      /*---- Custom Validation: Set Leaf Node flag  ----*/ 
      if (STD_HIER_PROD_LVL eq input(&g_max_PROD_LVL,best12.)) then do; 
         STD_HIER_LEAF_NODE_FLG = 1; 
      end;  /* if (STD_HIER_PROD_LVL eq input(&g_max_PROD_LVL,best12.)) */
      else do; 
         STD_HIER_LEAF_NODE_FLG = 0; 
      end;  /* else */
      
      /*---- Custom Validation: CHANGE_FLG  ----*/ 
            if (UPCASE(CHANGE_FLG) NOT IN ("&etl_insert_flg","&etl_update_flg","&etl_delete_flg")) then 
            do; 
               ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"%etl_msg(mid=5305,args=CHANGE_FLG)"),$300.); 
            end;  /* if (UPCASE(CHANGE_FLG) NOT IN ("&etl_insert_flg","&etl_update_flg","&etl_delete_flg")) */
            else 
            do; 
            end;  
      
      /*---- Custom Validation: Missing PROD_NM  ----*/ 
      if (MISSING(PROD_NM)) then do; 
         PROD_NM = put(PROD_ID,$32.); 
      end;
      
      /*---- Custom Validation: Change_flg=D (&etl_delete_flg) ----*/ 
      if (upcase(CHANGE_FLG) =upcase("&etl_delete_flg")) then do; 
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5302,args=%nrstr(Change_flg=&etl_delete_flg))"),$300.); 
      end;
      
      /*---- Custom Validation: Invalid PROD_LVL  ----*/ 
      if (NOT MISSING(STD_HIER_PROD_LVL) AND STD_HIER_PROD_LVL not in (&g_valid_prod_lvls.)) then do; 
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5305,args=PROD_LVL)"),$300.); 
      end;

      /*---- If solution have only std hierarchy then reject product record with null std. level ---- */
      if &cnt_assoc_cd = 1 and MISSING(STD_HIER_PROD_LVL) then do;
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5305,args=STD_HIER_PROD_LVL)"),$300.); 
      end;
 
      /*---- existing prod's DISC_DT is missing and same PROD_ID is coming in STG with diff intr_dt & CHANGE_FLG = I, 
             then validate PROD_INTR_DT of incoming record and then populate valid DISC_DT ----*/ 
      if (NOT MISSING(DM_SAL_INTR_DT ) & UPCASE(CHANGE_FLG) =UPCASE("&etl_insert_flg") AND (NOT MISSING(DM_SAL_DISC_DT) & 
          DM_SAL_DISC_DT le DM_SAL_INTR_DT)) 
      then do;
         chk_recycle_flg = 1;
      end;

	  /*---- Custom Validation: PROD_ID change at LVL 1 NOT ALLOWED. But update in Prod_nm, intr_dt etc are allowed.  ----*/ 
      if (&etls_recordsBefore1 gt 0 & STD_HIER_PROD_LVL= 1 & "&PROD_ID_1_LVL" ne %str("") and PROD_ID ne "&PROD_ID_1_LVL") then do; 
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5344)"),$300.); 
      end;

	  /*---- Modified: Custom Validation: Recycled product at lvl = 1 not allowed  ----*/ 
      if (NOT MISSING(PROD_HIER_SK)                   AND
          UPCASE(CHANGE_FLG) =UPCASE("&etl_insert_flg")       AND
          STD_HIER_PROD_LVL eq 1                      AND
          PROD_ID eq "&PROD_ID_1_LVL") then do;
          ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5344)"),$300.);
      end;
    

      /*---- Modified: Custom Validation: Restrict the future re-cycled products  ----*/ 
      if (NOT MISSING(PROD_HIER_SK)                 AND
          UPCASE(CHANGE_FLG) =UPCASE("&etl_insert_flg")     AND 
          NOT MISSING(PROD_INTRODUCTION_DT)         AND
          NOT MISSING(DM_SAL_INTR_DT )              AND
          NOT MISSING(DM_SAL_DISC_DT)               AND
          PROD_INTRODUCTION_DT gt "&App_dt") then do; 
          ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5377)"),$300.); 
        /* 5377: Future re-cycled products not allowed */
      end;
  
      /* Modified: The update PROD_ID could be present in this stage table. So recycled prod will be processed later */
      if (NOT MISSING(PROD_HIER_SK)         AND
          NOT MISSING(PROD_INTRODUCTION_DT) AND
          STD_HIER_PROD_LVL ne 1            AND
          NOT MISSING(DM_SAL_INTR_DT )      AND
          NOT MISSING(DM_SAL_DISC_DT)       AND 
          PROD_INTRODUCTION_DT le "&App_dt" AND
          UPCASE(CHANGE_FLG) eq UPCASE("&etl_insert_flg")) then do;
         chk_recycle_flg=1;            
      end;

	  /* Modified: Check with DM product : There should be gap of 1 day. */
      if (NOT MISSING(PROD_HIER_SK)         AND
          NOT MISSING(PROD_INTRODUCTION_DT) AND
          STD_HIER_PROD_LVL ne 1            AND
          NOT MISSING(DM_SAL_INTR_DT )      AND
          MISSING(DM_SAL_DISC_DT)           AND 
          PROD_INTRODUCTION_DT - 1 le DM_SAL_INTR_DT AND
          UPCASE(CHANGE_FLG) eq UPCASE("&etl_insert_flg")) then do;
			ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5348)"),$300.);
                   
      end;

      /* Modified: If a product exists in datamrt with missing intr_dt. and same product is coming in stg
            with missing INTR_DT, prod_id = ABC and CHANGE_FLG = I then, reject that record. */

      if (NOT MISSING(PROD_HIER_SK)                   AND 
          UPCASE(CHANGE_FLG) = UPCASE("&etl_insert_flg")      AND
          STD_HIER_PROD_LVL ne 1                      AND
          (MISSING(DM_SAL_INTR_DT ) OR (MISSING(PROD_INTRODUCTION_DT))))                   
      then do;
           ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5523)"),$300.);
           /* 5523: Cannot recycle product, either stg or dm PROD_INTRODUCTION_DT is missing/invalid */ 
      end;
      

      /*---- Modified: Custom Validation: Identify Recycled products  ----*/ 
      if (NOT MISSING(PROD_HIER_SK)                   AND
          UPCASE(CHANGE_FLG) =UPCASE("&etl_insert_flg")      AND
          NOT MISSING(PROD_INTRODUCTION_DT)           AND
          STD_HIER_PROD_LVL ne 1                      AND
          NOT MISSING(DM_SAL_INTR_DT )                AND
          NOT MISSING(DM_SAL_DISC_DT)                 AND 
          PROD_INTRODUCTION_DT gt DM_SAL_INTR_DT      AND
          PROD_INTRODUCTION_DT gt DM_SAL_DISC_DT) then do; 
         Call Missing (PROD_HIER_SK); 
      end;

 
      /*---- Custom Validation: PROD_DISCONTINUED_DT (in incr load) is less than PROD_INTRODUCTION_DT (that was loaded in an initial load)----*/ 
      if (MISSING(PROD_INTRODUCTION_DT)) and (NOT MISSING(PROD_DISCONTINUED_DT)) and 
           (NOT MISSING(DM_SAL_INTR_DT)) and (PROD_DISCONTINUED_DT le DM_SAL_INTR_DT) then do;           
          ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5331,args=STG.PROD_DISCONTINUED_DT DM.PROD_INTRODUCTION_DT)"),$300.);         
      end;


      /*---- Custom Validation: Invalid Recycled Prod ID.  ----*/ 
      if (NOT MISSING(PROD_HIER_SK)                   AND 
          UPCASE(CHANGE_FLG) = UPCASE("&etl_insert_flg")      AND
          NOT MISSING(DM_SAL_INTR_DT)                 AND
          (PROD_INTRODUCTION_DT le DM_SAL_INTR_DT OR PROD_INTRODUCTION_DT le DM_SAL_DISC_DT)) then do; 
           chk_recycle_flg=1;
      end;
      
      /*---- Custom Validation: PROD_INTRODUCTION_DT should be before PROD_DISCONTINUED_DT ----*/ 
      if (NOT MISSING(PROD_INTRODUCTION_DT) & 
           (NOT MISSING(PROD_DISCONTINUED_DT) & PROD_INTRODUCTION_DT ge PROD_DISCONTINUED_DT))
      then do;  
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5331,args=PROD_DISCONTINUED_DT PROD_INTRODUCTION_DT)"),$300.); 
      end;

     /*---- Custom Validation: Re-activation of discontinued products not allowed ----*/ 
     if (NOT MISSING(PROD_INTRODUCTION_DT) &
         (NOT MISSING(DM_SAL_DISC_DT) & UPCASE(CHANGE_FLG) = upcase("&etl_update_flg") & PROD_INTRODUCTION_DT ge DM_SAL_DISC_DT))
     then do;
        ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,"* %etl_msg(mid=5345)"),$300.); 
     end;             
      
      /*---- Custom Validation: Std_hier_prod_lvl does not match with stg_prod_hierarchy  ----*/ 
      if (NOT MISSING(STG_STD_PROD_LVL) & STD_HIER_PROD_LVL NE STG_STD_PROD_LVL) then 
      do; 
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5315,args=STD_HIER_PROD_LVL STG_PROD_HIERARCHY.prod_lvl)"),$300.); 
      end;
      
      /*---- Custom Validation: Std_hier_prod_lvl cannot change during incremental load ----*/ 
      if (NOT MISSING(DM_PROD_LVL) & STD_HIER_PROD_LVL NE DM_PROD_LVL) then 
      do; 
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,"* %etl_msg(mid=5346)"),$300.);
      end; 

      /*---- Custom Validation: Volume cannot be 0 if not null ----*/ 
      if (NOT MISSING(VOLUME) & VOLUME LE 0) then 
      do; 
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,"* %etl_msg(mid=5331,args=VOLUME 0)"),$300.); 
      end;

      /* if PROD_INTRODUCTION_DT is missing then fetch the existing value in Product_dm */
      if NOT MISSING(PROD_HIER_SK) and MISSING(PROD_INTRODUCTION_DT) then 
         PROD_INTRODUCTION_DT = DM_SAL_INTR_DT;

       /* if PROD_DISCONTINUED_DT is missing then fetch the existing value in Product_dm */

	    if NOT MISSING(PROD_HIER_SK) and MISSING(PROD_DISCONTINUED_DT) and NOT MISSING(PROD_INTRODUCTION_DT) and PROD_INTRODUCTION_DT gt DM_SAL_DISC_DT
           and PROD_INTRODUCTION_DT gt DM_SAL_INTR_DT and upcase(change_flg) = upcase("&etl_update_flg") then
         PROD_DISCONTINUED_DT = DM_SAL_DISC_DT;

       /*---- Custom Validation: PROD_INTRODUCTION_DT is Missing for lowest level nodes ----*/ 
      if (MISSING(PROD_INTRODUCTION_DT) AND STD_HIER_PROD_LVL eq input(&g_max_PROD_LVL,best12.)) then do; 
         ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5305,args=PROD_INTRODUCTION_DT)"),$300.); 
      end;

      if (UPCASE(CHANGE_FLG) = upcase("&etl_update_flg")) then do;
         if PREV_PROD_DISC_DT gt 0 and (PROD_INTRODUCTION_DT le PREV_PROD_DISC_DT) then
            ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5331,args=STG.Prod_intro_dt Previous_Prod_disc_dt)"),$300.); 
      end;   

      /*---- Custom Validation: Remove Invalid records  ----*/ 
      if (ETLS_EXCEPTION_COND eq "") then do; 
         /* move record to output file  */       
         CREATED_DTTM = "&etls_stepStartTime"dt; 
         if chk_recycle_flg eq 1 then 
            output work.CHECK_RECYCLED_PRODS;
         else 
            output work.STG_VALID_PRODUCT;   
      end;
      else do;
         ETL_ERROR_JOBRUNTIME = "&etls_stepStartTime"dt; 
         output work.etls_Errors;   /* move record to error file  */ 
      end;
      
   run; 
   
   %rcSet(&syserr);
   
   proc sql;
      create index prod_id on work.STG_VALID_PRODUCT;
   quit;   
      
   data work.CHECK_RECYCLED_PRODS (drop = etls_exception_cond etl_error_jobruntime)
        work.invalid_recycled_prods;
      length etls_exception_cond $300 etl_error_jobruntime 8;
      set work.CHECK_RECYCLED_PRODS;
      
      set work.STG_VALID_PRODUCT (keep=prod_id change_flg prod_introduction_dt prod_discontinued_dt
                                  rename=(change_flg=v_chg_flg prod_introduction_dt=v_prod_intr_dt prod_discontinued_dt=v_prod_disc_dt))
          key=prod_id;                        
          
      if _iorc_ eq 0 then do;
         if NOT (upcase(v_chg_flg)=upcase("&etl_update_flg") and 
                 prod_introduction_dt gt v_prod_intr_dt and
                 prod_introduction_dt gt v_prod_disc_dt) then
            
            etls_exception_cond = "%etl_msg(mid=5347)";
      end;
      else if (NOT MISSING(PROD_HIER_SK)                   AND 
               UPCASE(CHANGE_FLG) = upcase("&etl_insert_flg")      AND          
               NOT MISSING(DM_SAL_INTR_DT)                 AND
              (PROD_INTRODUCTION_DT le DM_SAL_INTR_DT OR PROD_INTRODUCTION_DT le DM_SAL_DISC_DT)) then do;
            ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5311)"),$300.);
      end;
      else if (MISSING(PROD_HIER_SK)                   AND
               UPCASE(CHANGE_FLG) = upcase("&etl_insert_flg")  AND 
               NOT MISSING(DM_SAL_INTR_DT)             AND
               NOT MISSING(DM_SAL_DISC_DT)             AND
               DM_SAL_INTR_DT ne DM_SAL_DISC_DT        AND
               prod_introduction_dt le DM_SAL_DISC_DT)
      then do;
            ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5348)"),$300.); 
      end;
      else if (NOT MISSING(DM_SAL_INTR_DT ) & UPCASE(CHANGE_FLG) =upcase("&etl_insert_flg") AND (NOT MISSING(DM_SAL_DISC_DT) & 
          DM_SAL_DISC_DT le DM_SAL_INTR_DT)) 
      then do;
          ETLS_EXCEPTION_COND = put(cats(ETLS_EXCEPTION_COND,'0a'x,"* %etl_msg(mid=5348)"),$300.); 
      end;

      if missing(etls_exception_cond) then output work.CHECK_RECYCLED_PRODS;
      else do;   
         ETL_ERROR_JOBRUNTIME = "&etls_stepStartTime"dt; 
         output work.invalid_recycled_prods;
      end;   
      _iorc_=0; _error_=0;
      drop v_chg_flg v_prod_intr_dt v_prod_disc_dt;
   run;   
   
   /* update the errors table  */ 
   proc append base = DI_MON.STG_PRODUCT_X 
               data = work.etls_Errors force nowarn; 
   run; 
   
   proc append base = DI_MON.STG_PRODUCT_X 
               data = invalid_recycled_prods force nowarn; 
   run;
   
         
   proc append base = work.STG_VALID_PRODUCT 
               data = CHECK_RECYCLED_PRODS force nowarn; 
   run;

   /* 
    If existing product's disc_dt is null and same PROD_ID is coming in STG with CHANGE_FLG = I and INTR_DT 
    is greater than existing product's intr_dt then, populate existing product's disc_dt = incoming prod_intr_dt -1 
   */
   
   proc sql;
      create index prod_id on work.update_prod_disc_dt;
   quit; 

   %if %di_util_nobs(Update_prod_disc_dt) gt 0 %then %do;
     data vld_data;
        merge Update_prod_disc_dt(in = a) Stg_valid_product(in = b);
        by prod_id PROD_INTRODUCTION_DT;
        if a and b;
     run;
   
     proc sql noprint;
        CREATE INDEX comp_key ON di_data.PRODUCT_DM (prod_id ,prod_introduction_dt);
     quit;

     data di_dm.product_dm;
        set vld_data;
        prod_introduction_dt=dm_sal_intr_dt;
        modify di_dm.product_dm key = comp_key;
  
        if _iorc_ in( %sysrc(_SOK)) then do; 
         prod_discontinued_dt = DM_SAL_DISC_DT;
        end;
        _iorc_ = 0; 
        _error_ = 0; 
     run;
     
    proc sql;
        drop index comp_key from di_data.product_dm;
     quit;

   %end;

   %rcSet(&syserr);
   %rcSet(&sqlrc); 
    
%mend etls_Validate;

%etls_Validate(etls_Source=work.stg_prod_details);

%rcSet(&syserr); 
%rcSet(&sqlrc); 

/*---- End of User Written Code  ----*/ 

/*---- Start of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            Table Loader                            A5EEB9WX.BI0000AA * 
 * Transform:       Table Loader (Version 2.1)                                * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Table:    Data Validation -                       A5EEB9WX.BK000093 * 
 *                   work.STG_VALID_PRODUCT                                   * 
 * Target Table:    PRODUCT_DM - DI_DM.PRODUCT_DM           A5EEB9WX.B90001KW * 
 *                                                                            * 
 * User Written:    SourceCode                                                * 
 *============================================================================*/ 

%let trans_rc = 0;
%let etls_stepStartTime = %sysfunc(datetime(), datetime20.); 

%let SYSLAST = %nrquote(work.STG_VALID_PRODUCT); 

%let ETLS_SYSLAST = &SYSLAST;
/*---- Start of Pre-Process Code  ----*/ 

%di_etl_keep_work_table(STG_VALID_PRODUCT);
/*---- End of Pre-Process Code  ----*/ 

%rcSet(&syserr); 
%rcSet(&sqlrc); 

%let SYSLAST = &ETLS_SYSLAST;

/*---- Start of User Written Code  ----*/ 

%etl_Scd1Loader
       (
       srctable   = WORK.STG_VALID_PRODUCT
      ,tgttable   = DI_DM.PRODUCT_DM
      ,buskey     = PROD_ID PROD_HIER_SK
      ,surkey     = PROD_HIER_SK
      ,rootnode_condn=%nrstr(STD_HIER_PROD_LVL=1)
      ,change_flg = change_flg
      ,sysopt     =
      );
      
%let etls_table = %nrquote(PRODUCT_DM);
%let etls_lib   = %nrquote(DI_DM);

data _null_; 
   call symput('etls_endTime',put(datetime(),datetime.));
run;

%etls_sendjobstatus(source_table = DIDM_STG.STG_PRODUCT);
/*---- End of User Written Code  ----*/ 

%rcSet(&syserr); 
%rcSet(&sqlrc); 

/*---- End of User Written Code  ----*/ 

/*============================================================================* 
 * Step:            Extract                                 A5W0V4GZ.BH000MKN * 
 * Transform:       Extract                                                   * 
 * Description:                                                               * 
 *                                                                            * 
 * Source Table:    PRODUCT_DM - DI_DM.PRODUCT_DM           A5W0V4GZ.BB000AJS * 
 * Target Table:    PROD_HIER_SK - DI_MON.PROD_HIER_SK      A5W0V4GZ.BB000ALQ * 
 *============================================================================*/ 


/*---- Map the columns  ----*/ 
proc datasets lib = DI_MON nolist nowarn memtype = (data view);
   delete PROD_HIER_SK;
quit;

%put %str(NOTE: Mapping columns ...);
proc sql;
   create table DI_MON.PROD_HIER_SK as
      select
         (MAX(PROD_HIER_SK)) as PROD_HIER_SK length = 6,
         PROD_ID
   from DI_DM.PRODUCT_DM
   group by
      PROD_ID
   ;
quit;

%let SYSLAST = DI_MON.PROD_HIER_SK;

%rcSet(&sqlrc); 

/*---- Start of Post-Process Code  ----*/ 

%let etls_endTime = %sysfunc(datetime(),datetime.);
/*---- End of Post-Process Code  ----*/ 

%rcSet(&syserr); 
%rcSet(&sqlrc); 

/**  Step end Extract **/
