/* 
SUPPORT: sinprs Swain, Pradeep
*/ 
/****************************************************************************************************************
                Demand Intelligence

NAME:           di_assigntranslibs.sas

DESCRIPTION:    Demand Intelligence initialization macro for Production


                
HISTORY:        March 26,2008 creation.

NOTES:          This macro is created as a way to initialize transaction libraries.
                When would this be used:
                  1) during a batch session initialization
                  2) A stored process server being brought up can call this library
                     if the connection to rdbms fails; this could be because after the
                     stored process server was brought up; the db2 engine could have
                     been brought down- this would result in the connections not being
                     active anymore
                     
                     transaction libraries are the following
                     
                     di_trans
                     di_t_dbc
                     di_dm2
                     
HISTORY:        March 26,2008 creation.
                Feb 07, 2018 Defect:S1373965: Libname statement getting truncated and hence error while 
		                     assigning di_dm2 library via sas batch - increasing the length of variables 
							 password, temp, temp2
******************************************************************************************************************/


%macro di_assigntranslibs()/*storesecure*/;

/* Find and assign the Transactional libraries */

data _null_ ;

   /* Find the information for our libraries in metadata and assign them */

  length svruri $200 srcconnuri $200 propuri $200 loginuri $200 dbname $200 userid $60 password $200;
  length schemauri $200 schemaname $200 id $ 17 vendor $ 20 server $ 200 uri $ 200 name $ 200 value $ 200 delim $ 200 ;
  length temp $ 400 temp2 $ 400 liburi $ 200 ;
  length libref $ 200 opts $ 200 ;

  userid=""; password="";
  svruri="omsobj:ServerComponent?@Name='&dbServerName'";

  rc=metadata_getattr(svruri, "ClassIdentifier", vendor);
  vendor = upcase(vendor) ;

  if vendor = 'DB2UNXPC' then vendor = 'DB2' ;
  else if vendor = 'ORACLE' then vendor = 'ORACLE' ;
  else vendor = '' ;

  /* Get the associated "SourceConnections" for this server */

  rc=metadata_getnass(svruri, "SourceConnections", 1, srcconnuri);
  if rc ne 1 then do;
    put "ERROR: Unable to locate source connection for &dbServerName.. Please confirm server name.";
    call symput("continue","NO");
    stop;
  end;

  /* Get all libraries associated with this source connection */

  rc=metadata_getnass(srcconnuri, "Libraries", 1, liburi);
  if rc lt 1 then do;
    put "No libraries found for &dbServerName to assign.";
    stop;
  end;

  n = rc ;

  /* Assign the libraries */

  do i = 1 to n ;
     rc=metadata_getnass(srcconnuri, "Libraries", i, liburi);	
     rc=metadata_getattr(liburi, "Libref", libref);
     libref = trim(libref) ;
     rc=metadata_getnass(liburi, "Properties", i, propuri);	
	 nprop = rc ;
	 opts = "" ;
     if nprop gt 0 then do ;
       do j = 1 to nprop ;
         rc=metadata_getnass(liburi, "Properties", j, propuri);	
         rc=metadata_getattr(propuri, "PropertyName", name);
         rc=metadata_getattr(propuri, "Delimiter", delim);
         rc=metadata_getattr(propuri, "DefaultValue", value);
         if trim(name) = 'OptionString' then
           opts = trim(opts) || " " || trim(value) ;
         else
           opts = trim(opts) || " " || trim(name) || trim(delim) || trim(value) ;
       end ;
     end ;
     
      if vendor = 'DB2' then
	     temp = "datasrc=&DI_TRANS_DATABASE schema=&DI_TRANS_SCHEMA user=&DI_TRANS_USER" ;
	   else if vendor = 'ORACLE' then
	   	temp = "path=&DI_TRANS_DATABASE schema=&DI_TRANS_SCHEMA user=&DI_TRANS_USER" ;
	   	
	    temp2 = trim(temp) || "  password=""&DI_TRANS_PASSWORD""" || " " || trim(opts) ;
       rc=libname(libref,,vendor,temp2) ;
       if rc ~= 0 then do;
           sysmsg = sysmsg();
           put sysmsg;
       end;
       else do ;
         put "Library " libref "was assigned using:" ;
		 put "LIBNAME " libref vendor temp "password=xxxxxx " opts ;
	   end ;
     end ;


run ;

%mend di_assigntranslibs;
